var dtMergeWindows =
{

  // select a tab for merging (by default assigned to ctrl-clicking on tab)
  selectBrowser: function(aTab)
  {
    if (aTab.hasAttribute('mergeselected')) {
      aTab.removeAttribute('mergeselected');
      aTab.label = aTab.label.substr(4);
    }
    else {
      aTab.setAttribute('mergeselected', 'true');
      aTab.label = "(*) "+aTab.label;
    }
  },


  // merge several windows to one window, or only selected tabs to previous focussed window,
  // or only current window with previous window
  mergeWindows: function()
  {
    var mergePopups = duplicateTab.prefService.getBoolPref('duplicatetab.merge.mergePopups');
    var placePopupNextToOpener = duplicateTab.prefService.getBoolPref('duplicatetab.merge.placePopupNextToOpener');
    var mergeAllWindows = duplicateTab.prefService.getBoolPref('duplicatetab.merge.mergeAllWindows');

    // other variables used in this function
    var currentIndex, tabs, browsers = new Array(), i, j;

    // get browser windows listed and add them to an array
    var windowsArrayList = dtMergeWindows.listWindows();
    var windowsArray = windowsArrayList.windowsArray;
    var popupsArray = windowsArrayList.popupsArray;
    var currentWindowIsPopup = windowsArrayList.isCurrentWindowPopup;

    // force the merging of windows when the current window is a popup
    // and the popup would be merged with one other (non popup) window
    var forceMergeWindows = (currentWindowIsPopup && (windowsArray.length == 1 || !mergeAllWindows));

    // foreground window is not a popup other window(s) is/are popup(s)
    var mergeBackGround = (windowsArray.length == 0 && popupsArray.length > 0 && !currentWindowIsPopup && mergePopups);

    // alert if just one window is open; no need to merge one window
    if (!forceMergeWindows && ((!mergePopups && windowsArray.length < 1 && !currentWindowIsPopup)
        || windowsArray.length+popupsArray.length < 1))
    {
      var errorMessage = duplicateTab.stringBundle.getString('duplicatetab.merge.error');
      var errorimage = "chrome://global/skin/icons/Warning.png"
      if (getBrowser().showMessage) {
        // Firefox 1.0 / 1.5
        getBrowser().showMessage(gBrowser.selectedBrowser, errorimage, errorMessage, '',
                                 null, null, null, "top", true);
      }
      else if (getBrowser().getNotificationBox) {
        // Firefox 2.0
        var notificationTopic = "duplicateTab:cannot-merge-one-window"
        var notificationBox = getBrowser().getNotificationBox();
        var notification = notificationBox.getNotificationWithValue(notificationTopic);
        if (notification)
          notification.label = errorMessage;
        else {
          var buttons = [];
          const priority = notificationBox.PRIORITY_INFO_MEDIUM;
          notificationBox.appendNotification(errorMessage, notificationTopic, errorimage, priority, buttons);
        }
      }
      else
        window.alert(errorMessage);
      return;
    }

    // check if one or more tabs are selected to be merged
    var selectedTabs = getBrowser().mTabContainer.getElementsByAttribute('mergeselected','true');
    var tabsSelected = (selectedTabs.length > 0);

    // fix for selecting the right tab after merging when tablib is enabled.
    var tablibRelative = false;
    if (gBrowser.moveTabTo && duplicateTab.tablibPref)
      tablibRelative = duplicateTab.prefService.getBoolPref(duplicateTab.tablibPref);

    // don't merge the current window if it is a popup and all windows are to be merged except popups
    var dontMergeCurrentWindow = (!forceMergeWindows && !mergePopups
                                   && currentWindowIsPopup && mergeAllWindows);

    // get currently focussed and previously focussed window
    // and get the window which has been out of focus longest
    var currentWindow = (dontMergeCurrentWindow)?windowsArray[windowsArray.length-1]:windowsArrayList.currentWindow;
    var previousWindow = (dontMergeCurrentWindow)?windowsArray[windowsArray.length-2]:windowsArray[windowsArray.length-1];
    var firstWindow = windowsArray[0];

    if (mergeBackGround) {
      // to correctly merge popup(s) when only the current window is not a popup
      currentWindow = popupsArray.pop();
      firstWindow = windowsArrayList.currentWindow;
      previousWindow = windowsArrayList.currentWindow;
      currentWindowIsPopup = true;
      windowsArray[0] = windowsArrayList.currentWindow;
    }

    var previousWindowTabBar = previousWindow.getBrowser().mTabContainer;
    var firstWindowTabBar = firstWindow.getBrowser().mTabContainer;

    var previousWindowTabIndex = (tablibRelative)?previousWindowTabBar.selectedIndex:previousWindowTabBar.childNodes.length;
    var firstWindowTabIndex = (tablibRelative)?firstWindowTabBar.selectedIndex:firstWindowTabBar.childNodes.length;

    // get currently focussed tab, so it can be selected again after merging
    if (tabsSelected && !gBrowser.mCurrentTab.hasAttribute('mergeselected'))
    {
      i = 0, j = -1;
      while (i < gBrowser.mTabContainer.childNodes.length)
      {
        if (gBrowser.mTabContainer.childNodes[i].hasAttribute('mergeselected'))
          j++;
        if ((i >= gBrowser.mTabContainer.selectedIndex) && (j > -1))
        {
          currentIndex = j;
          break;
        }
        i++;
      }
    }
    else
      currentIndex = (tabsSelected)?gBrowser.mTabContainer.selectedIndex:currentWindow.getBrowser().mTabContainer.selectedIndex;

    // If more then one window is opened and not all windows must be merged to one:
    // Either because of selected tabs or the pref is set to do so
    // Merge only the (selected) tabs of the currently focussed window and the previously focussed window
    if (tabsSelected || !mergeAllWindows) {
      var openerPosition = -1;
      tabs = currentWindow.getBrowser().mTabContainer.childNodes;

      // if tabs have been selected, list them in an array
      if (tabsSelected && !mergeBackGround)
        browsers = listTabs(selectedTabs, window);
      else if (tabsSelected && mergeBackGround) {
        var previousTabs = previousWindow.getBrowser().mTabContainer.childNodes;
        browsers = listTabs(tabs, currentWindow);
        for (i = 0; i < previousTabs.length; i++) {
          if (!previousTabs[i].hasAttribute('mergeselected'))
            previousTabs[i].setAttribute('mergeremove', 'true');
        }
      }

      // if no tabs have been selected, list all tabs of the window in an array
      else if (!tabsSelected) {
        browsers = listTabs(tabs, currentWindow);
        browsers = removeDuplicates(browsers, previousWindow);
      }

      // if the current window is a popup window place the popup next to its opener
      // when that opener is found in the previous window.
      if (currentWindowIsPopup && placePopupNextToOpener)
      {
        var openerPositionArray = getPopupOpenerPosition(currentWindow, [previousWindow], previousWindowTabIndex);
        openerPosition = openerPositionArray[0];
        if (openerPositionArray[1] == 'tabs')
        {
          currentIndex = (mergeBackGround)? currentIndex: openerPosition + currentIndex;
          // the popup opener was found in one of the tabs in the window that will remain.
          // place the tab(s) next to the opener
          setTabsInBrowser(previousWindow, browsers, openerPosition);
        }
      }

      // place all tabs in one window
      if (openerPosition == -1) {
        currentIndex = previousWindowTabIndex + currentIndex;
        setTabsInBrowser(previousWindow, browsers);
      }

      // prompt a warning if some tabs are selected and the window with remaining tabs is closed
      var promptpref = duplicateTab.prefService.getBoolPref('duplicatetab.merge.promptOnClose');
      var closepref = duplicateTab.prefService.getBoolPref('duplicatetab.merge.closeOnSelect');
      var askForPrompt = ((promptpref) && (tabsSelected) && (selectedTabs.length < tabs.length));
      var promptOK = false, promptAgain = { value:true };
      if (askForPrompt && closepref) {
        var promptService = Components.classes["@mozilla.org/embedcomp/prompt-service;1"].getService(Components.interfaces.nsIPromptService);
        var promptTitle = duplicateTab.stringBundle.getString('duplicatetab.merge.warning.title');
        var promptMessage = duplicateTab.stringBundle.getString('duplicatetab.merge.warning.message');
        var promptCheckboxLabel = duplicateTab.stringBundle.getString('duplicatetab.merge.warning.checkboxLabel');
        promptOK = promptService.confirmCheck(window, promptTitle, promptMessage, promptCheckboxLabel, promptAgain);
      }

      // select the tab which was selected in the top window
      previousWindowTabBar.selectedIndex = currentIndex;

      // close current window if all tabs are merged or if specified by user to do so with some tabs selected
      if ((promptOK || (!askForPrompt)) && closepref) {
        if (!promptAgain.value) {
          duplicateTab.prefService.setBoolPref('duplicatetab.merge.promptOnClose', false);
        }
        previousWindow.focus();
        currentWindow.close();
      }

      // If the window is not to be closed, remove the tabs that have been merged
      else {
        for (i = 0; i < selectedTabs.length; i++) {
          currentWindow.getBrowser().removeTab(selectedTabs.item(i));
        }
        previousWindow.focus();
      }

      if (mergeBackGround && tabsSelected) {
        // because the other windows are added to the window with selected tabs, the not selected tabs
        // have to be removed. (normally the selected tabs will be removed and placed in another window)
        for (i = 0; i < selectedTabs.length; i++) {
          selectedTabs.item(i).label = selectedTabs.item(i).label.substr(4);
        }

        for (i = 0; i < previousTabs.length; i++) {
          if (previousTabs[i].hasAttribute('mergeremove')) {
            if (i < currentIndex)
              currentIndex--;
            previousWindow.getBrowser().removeTab(previousTabs[i]);
          }
          if (previousTabs[i].hasAttribute('mergeselected'))
            previousTabs[i].removeAttribute('mergeselected');
        }
      }
      previousWindowTabBar.selectedIndex = currentIndex;
    }

    // Merge all windows to one window if more then one window is opened and
    // no tabs are selected and the pref is set to merge all windows.
    else if (!tabsSelected && mergeAllWindows)
    {
      // list all tabs on all windows with history scrollposition and textzoom
      var addedTabsLength = 0, popupsList = new Array();

      for (i = 1; i < windowsArray.length; i++)
      {
        tabs = windowsArray[i].getBrowser().mTabContainer.childNodes;
        browsers = browsers.concat(listTabs(tabs, windowsArray[i]));
        // the last window in the array is only the current window if merging of popups is disabled
        // and the current window is a popup window
        if (windowsArray[i] != currentWindow)
          addedTabsLength += tabs.length;
      }
      // the current window is a popup and popups should also be merged
      // or the current window is not a popup
      if (!dontMergeCurrentWindow)
      {
        tabs = currentWindow.getBrowser().mTabContainer.childNodes;
        if (!currentWindowIsPopup)
        {
          browsers = browsers.concat(listTabs(tabs, currentWindow));
          currentIndex = firstWindowTabIndex + addedTabsLength + currentIndex;
        }
        else
        {
          var listedTabs = listTabs(tabs, currentWindow);
          if (placePopupNextToOpener)
          {
            var openerPositionArray = getPopupOpenerPosition(currentWindow, windowsArray, firstWindowTabIndex);
            var openerPosition = openerPositionArray[0];
            switch (openerPositionArray[1])
            {
              case 'browsers':
                // the popup opener was found in the list of browsers that will be added to the remaining window.
                currentIndex = openerPosition + 1 + currentIndex;
                popupsList[openerPosition] = listedTabs;
                currentIndexSet = true;
                break;
              case 'tabs':
                // the popup opener was found in one of the tabs in the window that will remain.
                setTabsInBrowser(firstWindow, listedTabs, openerPosition);
                // in this case current index is fixed because the to be focused tab
                // has now already been added to the remaining window
                currentIndex = openerPosition + 1 + currentIndex;
                break;
              case 'end':
              default:
                // no popup opener was found, the popup will be added at the end of the tab list.
                browsers = browsers.concat(listedTabs);
                currentIndex = firstWindowTabIndex + addedTabsLength + currentIndex;
                break;
            }
          }
          else
          {
            browsers = browsers.concat(listedTabs);
            currentIndex = firstWindowTabIndex + addedTabsLength + currentIndex;
          }
        }
      }
      else // the current window is a popup and should not be merged.
        currentIndex = firstWindowTabIndex + addedTabsLength + currentIndex;

      // add popup windows to the remaining window when popup windows should also be merged.
      if (mergePopups)
      {
        for (i = 0; i < popupsArray.length; i++)
        {
          tabs = popupsArray[i].getBrowser().mTabContainer.childNodes;
          if (placePopupNextToOpener)
          {
            var listedTabs = listTabs(tabs, popupsArray[i]);
            var newWindowsArray = windowsArray;
            if (!currentWindowIsPopup && !mergeBackGround)
              newWindowsArray.push(currentWindow);
            var openerPositionArray = getPopupOpenerPosition(popupsArray[i], newWindowsArray, firstWindowTabIndex);
            var openerPosition = openerPositionArray[0];
            switch (openerPositionArray[1])
            {
              case 'browsers':
                // the popup opener was found in the list of browsers that will be added to the remaining window.
                if (openerPosition < currentIndex)
                  currentIndex += tabs.length;
                if (popupsList[openerPosition] instanceof Array)
                  popupsList[openerPosition] = popupsList[openerPosition].concat(listedTabs);
                else
                  popupsList[openerPosition] = listedTabs;
                break;
              case 'tabs':
                // the popup opener was found in one of the tabs in the window that will remain.
                setTabsInBrowser(firstWindow, listedTabs, openerPosition);
                if (openerPosition < currentIndex)
                  currentIndex += tabs.length;
                break;
              case 'end':
              default:
                // no popup opener was found, the pop up will be added at the end of the tab list.
                browsers = browsers.concat(listedTabs);
                break;
            }
          }
          else {
            browsers = browsers.concat(listTabs(tabs, popupsArray[i]));
          }
        }
      }

      browsers = removeDuplicates(browsers, firstWindow);

      // place all tabs in one window
      setTabsInBrowser(firstWindow, browsers);

      // add popups which have openers next to their openers.
      while (popupsList.length > 0) {
        var popupTabs = popupsList.pop();
        if (popupTabs instanceof Array) {
          var position = popupsList.length;
          setTabsInBrowser(firstWindow, popupTabs, position);
        }
      }

      // select the tab which was selected in the top window
      firstWindowTabBar.selectedIndex = currentIndex;

      // focus the window now containing the tabs and close all other windows
      firstWindow.focus();

      // close windows from where tabs have been merged
      for (i = 1; i < windowsArray.length; i++) {
        windowsArray[i].close();
      }

      // close popup windows if the popups have been merged
      if (mergePopups) {
        for (i = 0; i < popupsArray.length; i++) {
          popupsArray[i].close();
        }
      }

      // if current window has been merged close it
      if (!dontMergeCurrentWindow)
        currentWindow.close();
      // give focus again to the current window if it is a popup and popups should not be merged
      if (dontMergeCurrentWindow)
        setTimeout(windowsArrayList.currentWindow.focus, 0);
    }

    // retrieve the opener for a popup window, so it can be placed next to it
    function getPopupOpenerPosition(popupWindow, otherWindows, containerWindowTabsIndex)
    {
      var i, j, openerIndex = -1;
      var openerWindow = popupWindow.getBrowser().contentWindow.opener
      if (!openerWindow) {
        return [openerIndex, 'end'];
      }
      for (i = 0; i < otherWindows.length; i++) {
        if (otherWindows[i] == popupWindow)
          continue;
        var tabs = otherWindows[i].getBrowser().mTabContainer.childNodes;
        for (j = 0; j < tabs.length; j++)
        {
          openerIndex++;
          var possibleOpener = otherWindows[i].getBrowser().getBrowserForTab(tabs[j]).contentWindow;
          if (possibleOpener == openerWindow)
          {
            var where = (openerIndex >= containerWindowTabsIndex)? 'browsers': 'tabs';
            return [openerIndex, where];
          }
        }
      }
      return [-1, 'end'];
    }

    // list sessionhistory, scrollposition and textzoom of tabs in an array.
    function listTabs(tabs, aWindow)
    {
      var x = 0, curBrowser, browsersArray = new Array();
      for (x; x < tabs.length; x++) {
        curBrowser = aWindow.getBrowser().getBrowserForTab(tabs[x]);
        browsersArray[x] = [duplicateTab.copyTabHistory(curBrowser.webNavigation.sessionHistory),
                      curBrowser.contentWindow.scrollX, curBrowser.contentWindow.scrollY,
                      curBrowser.markupDocumentViewer.textZoom];
        // Support for Reveal extension: clone thumbnails
        if (curBrowser.thumbcache)
          browsersArray[x].push(duplicateTab.cloneThumbcache(curBrowser));
      }

      return browsersArray;
    }

    // add cloned tabs to a browser window
    function setTabsInBrowser(aWindow, pages)
    {
      var y = 0, openedBrowser, newTab;

      // add tabs next to currently selected tab
      if (tablibRelative)
      {
        var index = (aWindow == firstWindow)?firstWindowTabIndex:previousWindowTabIndex;
      }

      // add tabs to defined position in case of a popup: next to its opener
      var addToPosition = false;
      if (typeof(arguments[2]) == 'number')
      {
        addToPosition = true;
        var openerPosition = arguments[2];
        var position = arguments[2];
      }

      while (y < pages.length)
      {
        newTab = aWindow.getBrowser().addTab();
        if (addToPosition && getBrowser().moveTabTo)
        {
          var newTabPos = (aWindow.getBrowser().getTabIndex)? aWindow.getBrowser().getTabIndex(newTab): newTab._tPos;

          // the popup is placed next to its opener
          var newPosition = position + 1;
          if (newTabPos < newPosition)
            newPosition--;
          if (newTabPos != newPosition)
            aWindow.getBrowser().moveTabTo((aWindow.getBrowser().getTabIndex)? newTabPos: newTab, newPosition);

          // set the opener again for the popup (this data will otherwise be lost)
          var openerBrowser = aWindow.getBrowser().getBrowserForTab(aWindow.getBrowser().mTabContainer.childNodes[openerPosition]);
          aWindow.getBrowser().getBrowserForTab(newTab).contentWindow.opener = openerBrowser.contentWindow;
          position++;
        }
        else if (tablibRelative)
        {
          var firstArgument = (aWindow.getBrowser().getTabIndex)? aWindow.getBrowser().getTabIndex(newTab): newTab;
          aWindow.getBrowser().moveTabTo(firstArgument, index + 1);
          index++;
        }

        openedBrowser = aWindow.getBrowser().getBrowserForTab(newTab);
        duplicateTab.setClonedContent(openedBrowser, pages[y]);
        y++;
      }
    }

    function removeDuplicates(newTabs, aWindow)
    {
      const REMOVE_NO_DUPLICATES = 0;
      const REMOVE_SINGLE_HISTORY_DUPLICATES = 1;
      const REMOVE_ALL_DUPLICATES = 2;
      const REMOVE_MULTIPLE_HISTORY_DUPLICATES_ASK = 3;
      const REMOVE_ALL_DUPLICATES_ASK = 4;

      var removeDuplicates = REMOVE_NO_DUPLICATES;
      try
      {
        removeDuplicates = duplicateTab.prefService.getIntPref('duplicatetab.merge.removeDuplicates');
      }
      catch(e) {}

      var tabCount = 1, removeItem = null, duplicateRemoved = false, originalRemoved = false;
      if (removeDuplicates)
      {
        var originalTabs = aWindow.getBrowser().mTabContainer.childNodes;
        var tabLength = originalTabs.length;
        var allTabs = new Array();
        for (var i = 0; i < tabLength; i++) {
          var originalBrowser = aWindow.getBrowser().getBrowserForTab(originalTabs[i]);
          var originalTabURI = originalBrowser.currentURI.spec;
          var originalHistoryLength = originalBrowser.webNavigation.sessionHistory.count;
          allTabs.push([originalTabURI, originalHistoryLength]);
        }
        allTabs = allTabs.concat(newTabs);

        outerLoop:
          while (tabCount < allTabs.length)
          {
            var tabURI = (tabCount < tabLength)?allTabs[tabCount][0]:allTabs[tabCount][0][allTabs[tabCount][0].index].URI.spec;
            var tabHistoryLength = (tabCount < tabLength)?allTabs[tabCount][1]:allTabs[tabCount][0].length;

            innerLoop:
              for (var compareTab = 0; compareTab < tabCount; compareTab++)
              {
                var comparedTabURI = (compareTab < tabLength)?allTabs[compareTab][0]:allTabs[compareTab][0][allTabs[compareTab][0].index].URI.spec;
                var comparedTabHistoryLength = (compareTab < tabLength)?allTabs[compareTab][1]:allTabs[compareTab][0].length;

                if (comparedTabURI == tabURI)
                {
                  switch (removeDuplicates)
                  {
                    case REMOVE_SINGLE_HISTORY_DUPLICATES:
                      if (tabHistoryLength == 1)
                      {
                        if (tabCount < tabLength)
                          originalRemoved = true;
                        else
                          duplicateRemoved = true;
                        removeItem = tabCount;
                      }
                      else if (comparedTabHistoryLength == 1)
                      {
                        if (compareTab < tabLength)
                          originalRemoved = true;
                        else
                          duplicateRemoved = true;
                        removeItem = compareTab;
                      }
                      break innerLoop;

                    case REMOVE_ALL_DUPLICATES:
                      if (comparedTabHistoryLength == 1 && tabHistoryLength > 1)
                      {
                        if (compareTab < tabLength)
                          originalRemoved = true;
                        else
                          duplicateRemoved = true;
                        removeItem = compareTab;
                      }
                      else
                      {
                        if (tabCount < tabLength)
                          originalRemoved = true;
                        else
                          duplicateRemoved = true;
                        removeItem = tabCount;
                      }
                      break innerLoop;

                    case REMOVE_MULTIPLE_HISTORY_DUPLICATES_ASK:
                      // not implemented yet
                      break;

                    case REMOVE_ALL_DUPLICATES_ASK:
                      // not implemented yet
                      break;

                    default:
                      break;
                  }
                }
              }
            /*** END Compare statement ***/

            if (originalRemoved)
            {
              currentIndex--;
              aWindow.getBrowser().removeTab(originalTabs[removeItem]);
              allTabs.splice(removeItem, 1);
              tabLength--;
              removeItem = null;
              originalRemoved = false;
              continue outerLoop;
            }

            else if (duplicateRemoved)
            {
              allTabs.splice(removeItem, 1);
              removeItem = removeItem-tabLength;
              if ((removeItem <= currentIndex)  && (currentIndex > 0))
                currentIndex--;
              newTabs.splice(removeItem, 1);
              removeItem = null;
              duplicateRemoved = false;
              continue outerLoop;
            }
            y++;
          }
        /*** END Add statement ***/
      }

      return newTabs;
    }
  },

  // list all browser windows in een array from front to back (z-order on windows, on linux opening order);
  listWindows: function()
  {
    // function to check if a window is a popup or a normal window
    function isPopupWindow(domWin)
    {
      // the chrome flags are defined in the nsIWebBrowserChrome interface
      const webBrowserChrome = Components.interfaces.nsIWebBrowserChrome;

      var chromeBrowser = domWin.getBrowser().docShell.QueryInterface(Components.interfaces.nsIDocShellTreeItem)
                                .treeOwner.QueryInterface(Components.interfaces.nsIInterfaceRequestor)
                                .getInterface(webBrowserChrome);

      // check if all chrome is enabled in the window, using chromeFlags from nsIWebBrowserChrome
      var chromeAll = chromeBrowser.chromeFlags & webBrowserChrome.CHROME_ALL;
      if (chromeAll == webBrowserChrome.CHROME_ALL) {
        // no popup window (the window has all chrome enabled)
        return false;
      }

      // in other cases some chrome is missing from the window: popup window
      return true;
    }

    var windowsArray = new Array(), popupsArray = new Array();
    var i = 0, winEnumerator, currentWindow, isCurrentWindowPopup, win;

    var windowsMediator = Components.classes["@mozilla.org/appshell/window-mediator;1"].getService(Components.interfaces.nsIWindowMediator);
    currentWindow = windowsMediator.getMostRecentWindow('navigator:browser');

    // on linux z enumerator does not work: use normal enumerator
    var platform = currentWindow.navigator.platform;
    if (platform.indexOf('Linux') != -1)
    {
      winEnumerator = windowsMediator.getEnumerator('navigator:browser');
      while (winEnumerator.hasMoreElements())
      {
        win = winEnumerator.getNext();
        // list the currentWindow apart
        if (win == currentWindow.QueryInterface(Components.interfaces.nsIDOMWindow)) {
          currentWindow = win;
          isCurrentWindowPopup = isPopupWindow(win);
          continue;
        }
        if (isPopupWindow(win))
          popupsArray.push(win);
        else
          windowsArray.push(win);
      }
    }
    else
    {
      // use getZOrderEnumerator on Windows and Mac; it does not work on linux
      winEnumerator = windowsMediator.getZOrderDOMWindowEnumerator('navigator:browser', false);
      while (winEnumerator.hasMoreElements())
      {
        win = winEnumerator.getNext();
        // list the current window apart
        if (win == currentWindow.QueryInterface(Components.interfaces.nsIDOMWindow))
        {
          currentWindow = win;
          isCurrentWindowPopup = isPopupWindow(win);
          continue;
        }
        if (isPopupWindow(win))
          popupsArray.push(win);
        else {
          windowsArray.push(win);
        }
      }
    }
    var windowsList = {windowsArray: windowsArray, popupsArray: popupsArray, currentWindow: currentWindow, isCurrentWindowPopup: isCurrentWindowPopup};
    return windowsList;
  },

  // check if Tab Clicking Options extension is installed for assigning the select tab for merging event
  // and if it is the first time it is run together with Duplicate Tab
  checkTabClicking: function() {
    if ((duplicateTab.prefService.getPrefType("extensions.tabclicking.action") != 0) && (duplicateTab.prefService.getBoolPref("duplicatetab.with-tabclicking.firsttime")) && (window.tabClicking != undefined)) {

      // assign selecting tabs for merging to ctrl-click in Tab Clicking Options extension
      var tabclickingPrefs = duplicateTab.prefService.getCharPref("extensions.tabclicking.action");
      var mergeActionNum = (window.tabClicking.version != undefined)?'A':'9';
      if (tabclickingPrefs.indexOf(mergeActionNum) == -1) {
        var tcprefs1 = tabclickingPrefs.substr(0, 4);
        var tcprefs2 = tabclickingPrefs.substr(5);
        tabclickingPrefs = tcprefs1+mergeActionNum+tcprefs2;
        duplicateTab.prefService.setCharPref("extensions.tabclicking.action", tabclickingPrefs);

        // never perform this action again (only when both extensions are run together for the first time)
        duplicateTab.prefService.setBoolPref("duplicatetab.with-tabclicking.firsttime", false);
      }
    }
  },

  // check if Tab Clicking Options is installed, if not: check if ctrl key is pressed on a tab click
  // to select a tab for merging.
  tabBarClick: function(evt)
  {
    if ((window.tabClicking != undefined)
        || (evt.button != 0) || ((!evt.ctrlKey) && (!evt.metaKey))
        || (evt.originalTarget.getAttribute('class').indexOf('close') != -1))
      return;

    var tabClicked = false, evtTarget = evt.originalTarget;
    while (evtTarget.localName != 'tabs')
    {
      tabClicked = (evtTarget.localName == 'tab')
      if (tabClicked)
        break;
      evtTarget = evtTarget.parentNode;
    }

    if (tabClicked)
      dtMergeWindows.selectBrowser(evtTarget);
  }

}
